package com.cosmos.candelabra.data.remote.api.yahoofinance

import okhttp3.FormBody
import org.jsoup.Jsoup
import javax.inject.Inject
import javax.inject.Singleton

@Singleton
class YahooInitializer @Inject constructor(
    private val yahooFinance: YahooFinance,
    private val yahooConsent: YahooConsent,
    private val yahooCrumb: YahooCrumb
) {

    val isInitialized: Boolean
        get() = yahooCrumb.crumb != null

    suspend fun init(): Result<String?> {
        return runCatching {
            val body = yahooFinance.load()

            val doc = Jsoup.parse(body.string())
            val csrfToken = doc.selectFirst("input[name=csrfToken]")?.attr("value")

            csrfToken?.let { token ->
                val sessionId = doc.selectFirst("input[name=sessionId]")?.attr("value")
                val originalDoneUrl = doc.selectFirst("input[name=originalDoneUrl]")?.attr("value")
                val namespace = doc.selectFirst("input[name=namespace]")?.attr("value")

                val requestBody = FormBody.Builder()
                    .add("csrfToken", token)
                    .add("sessionId", sessionId.orEmpty())
                    .addEncoded("originalDoneUrl", originalDoneUrl.orEmpty())
                    .add("namespace", namespace.orEmpty())
                    .add("reject", "reject")
                    .build()

                yahooConsent.collectConsent(requestBody, sessionId.orEmpty())
            }

            yahooCrumb.fetchCrumb()
        }
    }
}
